/****************************************************************************

 This file is part of the qdu distribution
 Copyright (C) 2002, 2003, 2004  Gilles Debunne (Gilles.Debunne@imag.fr)
 Version 2.2-2, packaged on Tuesday February 1, 2005.

 http://artis.imag.fr/Membres/Gilles.Debunne/Code/QDU

 QDU is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 QDU is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with QDU; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifndef INCLUDE_DIRECTORY_VIEW_H
#define INCLUDE_DIRECTORY_VIEW_H

#include "item.h"

#include <qlistview.h>
#include <qvaluevector.h>
#include <qptrlist.h>

class DirectoryView : public QListView
{
  Q_OBJECT
  
public:
  DirectoryView( QWidget *parent = 0, const char *name = 0 );
  virtual ~DirectoryView();
  
signals:
  void scaleChanged(int s);
  
public slots:
  void setScale(int s);
  void fitScale();
  void sliderMinThreshold(int m);
  void setFileType(int i);
  void toggleHidden(bool);
  void setRootDirectory(const QString& rootDir);
  void refresh();
  void goUpOneLevel();
  void undo();
  void redo();
  bool renameItem(Item* item);
  virtual void clearSelection();

  float scale() { return scale_; }
  unsigned short int typeFilter() { return typeFilter_; };
  QDU_UINT64 minThreshold() { return minThreshold_; };
  bool showHidden() { return showHidden_; };
      
protected:
  void keyPressEvent( QKeyEvent * );

  // Drag and drop associated functions
  void contentsDragEnterEvent( QDragEnterEvent *e );
  void contentsDragMoveEvent( QDragMoveEvent *e );
  void contentsDragLeaveEvent( QDragLeaveEvent *e );
  void contentsDropEvent( QDropEvent *e );
  void contentsMouseMoveEvent( QMouseEvent *e );
  void contentsMousePressEvent( QMouseEvent *e );
  void contentsMouseReleaseEvent( QMouseEvent *e );
  
protected slots:
  void buttonPressed( QListViewItem* item, const QPoint & pnt, int c );
  void contextMenu( QListViewItem *item, const QPoint &, int );
  void openFolder();
  
private:
  bool applyToSelectedFiles(QDropEvent::Action action, Item* destDir, bool overWrite = true);
  void deleteSelectedFiles();
  void hideSelection(bool hide = true);
  void initTypeExtension();
  void updateStatusBar();
  void removeFromSelection(Item* item);
  bool garbageDirectoryIsOk();
  bool performAction(QDropEvent::Action action, Item* sourceDir, Item* item, Item* targetDir, const QString& newName);
  QDU_UINT64 maxVisibleSize() const;

  static bool directoryIsWritable(const QString& dir);
  static QString actionString(const QDropEvent::Action& action);

  // Current selection
  QPtrList<Item> selectedItems_;
  bool onlyOneItemIsSelected_;

  // Virtual garbage Item. 
  Item* garbage;

  // Whether hidden files are shown or not
  bool showHidden_;
  // Minimum visible file size
  QDU_UINT64 minThreshold_;
  // Histogram scale
  float scale_;
  unsigned short int typeFilter_;

  ////////////   D r a g   a n d   d r o p      /////////
  QListViewItem *dropItem;
  QTimer* autoOpenTimer;
  QPoint pressPos;
  bool mousePressed;

  ///////////    U n d o   -   r e d o     //////////
  struct UndoEvent
  {
    Item *sourceDir, *source, *targetDir;
    QDropEvent::Action action;
    QString prevName, newName;
  };
  QValueVector< QValueVector<UndoEvent> > undoHistory_;
  unsigned int undoIndex_, maxUndoIndex_;

  void beginUndoEvent();
  void endUndoEvent();
  void addEventInUndoHistory(QDropEvent::Action a, Item* sd, Item* s, Item* td, const QString& pn, const QString& nn );

private slots:
  void updateSelectedItemsList();
};
#endif // INCLUDE_DIRECTORY_VIEW_H
