/****************************************************************************

 This file is part of the qdu distribution
 Copyright (C) 2002, 2003, 2004  Gilles Debunne (Gilles.Debunne@imag.fr)
 Version 2.2-2, packaged on Tuesday February 1, 2005.

 http://artis.imag.fr/Membres/Gilles.Debunne/Code/QDU

 QDU is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 QDU is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with QDU; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#include "directoryView.h"

#include <iostream>

#include <qapplication.h>
#include <qdragobject.h>
#include <qheader.h>
#include <qtimer.h>
#include <qmessagebox.h>

using namespace std;

static const int autoopenTime = 750;

void DirectoryView::contentsDragEnterEvent( QDragEnterEvent *e )
{
  if ( !QUriDrag::canDecode(e) )
    {
      e->ignore();
      return;
    }

  hideSelection();

  QListViewItem *i = itemAt( contentsToViewport(e->pos()) );
  if ( i )
    {
      dropItem = i;
      dropItem->setSelected( true );
      autoOpenTimer->start( autoopenTime );
    }
  triggerUpdate();
}

void DirectoryView::contentsDragMoveEvent( QDragMoveEvent *e )
{
  if ( !QUriDrag::canDecode(e) )
    {
      e->ignore();
      return;
    }
  
  QPoint vp = contentsToViewport( e->pos() );
  QListViewItem *i = itemAt( vp );
  if ( i )
    {
      if ( i != dropItem )
	{
	  autoOpenTimer->stop();
	  i->setSelected( true );
	  if (dropItem)
	    dropItem->setSelected( false );
	  dropItem = i;
	  autoOpenTimer->start( autoopenTime );
	  triggerUpdate();
	}
      e->acceptAction();	
    }
  else
    {
      if ( dropItem )
	{
	  dropItem->setSelected( false );
	  triggerUpdate();
	}
      e->ignore();
      autoOpenTimer->stop();
      dropItem = NULL;
    }
}

void DirectoryView::contentsDragLeaveEvent( QDragLeaveEvent * )
{
  autoOpenTimer->stop();
  
  if (dropItem)
    {
      dropItem->setSelected( false );
      dropItem = NULL;
    }

  hideSelection(false);
  triggerUpdate();
}

void DirectoryView::contentsDropEvent( QDropEvent *e )
{
  autoOpenTimer->stop();

  if ( !QUriDrag::canDecode(e) )
    {
      e->ignore();
      return;
    }

  Item *targetDir = (Item*)(itemAt( contentsToViewport(e->pos()) ));

  // Go up to directory if dropped on a file
  if (targetDir && !targetDir->isDirectory)
    targetDir = (Item*)(targetDir->parent());

  if (e->source() != viewport())
    {
      QMessageBox::information(this, "Not yet implemented", "Drag and drop between two instances of qdu not (yet) implemented");
      e->ignore();
      return;
    }

  if ( targetDir )
    {
      // Temporary
      if (e->action() == QDropEvent::Move)
	applyToSelectedFiles(e->action(), targetDir);
      else
	QMessageBox::information(this, "Not yet supported", "Copying and linking files is not (yet) supported");
      e->acceptAction();
    }
  else
    e->ignore();
}

void DirectoryView::contentsMousePressEvent( QMouseEvent* e )
{
  switch (e->button())
    {
    case Qt::LeftButton :
      QListView::contentsMousePressEvent(e);
      break;
    case Qt::MidButton :
      {
	QPoint p( contentsToViewport( e->pos() ) );
	QListViewItem *i = itemAt( p );
	if ( i && i->isSelected() )
	  {
	    // if the user clicked into the root decoration of the item, don't try to start a drag!
	    if ( p.x() > header()->cellPos( header()->mapToActual( 0 ) ) +
		 treeStepSize() * ( i->depth() + ( rootIsDecorated() ? 1 : 0) ) + itemMargin() ||
		 p.x() < header()->cellPos( header()->mapToActual( 0 ) ) )
	      {
		pressPos = e->pos();
		mousePressed = true;
	      }
	  }
	break;
      }
    default :
      break;
    }
}

void DirectoryView::contentsMouseMoveEvent( QMouseEvent* e )
{
  switch (e->state() & Qt::MouseButtonMask)
    {
    case Qt::LeftButton :
      QListView::contentsMouseMoveEvent(e);
      break;
    case Qt::MidButton :
      if ( mousePressed && ( pressPos - e->pos() ).manhattanLength() > QApplication::startDragDistance() )
	{
	  mousePressed = false;
	  QListViewItem *item = itemAt( contentsToViewport(pressPos) );
	  if ( item && item->isSelected() )
	    {
	      QUriDrag* ud = new QUriDrag(viewport());
	      // ud->setUnicodeUris( selectionList );
	      ud->drag();
	      
	      // if (ud->target() != viewport()) DnD between two different qdu instances
	      // Requires code to update selectedItems_ 
	    }
	}
    default :
      break;
    }
}

void DirectoryView::contentsMouseReleaseEvent( QMouseEvent* e )
{
  switch (e->button())
    {
    case Qt::LeftButton :
      QListView::contentsMouseReleaseEvent(e);
      break;
    case Qt::MidButton :
      mousePressed = false;
    default :
      break;     
    }
}
