/****************************************************************************

 This file is part of the qdu distribution
 Copyright (C) 2002, 2003, 2004  Gilles Debunne (Gilles.Debunne@imag.fr)
 Version 2.2-2, packaged on Tuesday February 1, 2005.

 http://artis.imag.fr/Membres/Gilles.Debunne/Code/QDU

 QDU is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 QDU is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with QDU; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#include "item.h"
#include "directoryView.h"

#include <iostream>

#include <qdir.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qprogressdialog.h>
#include <qapplication.h>

using namespace std;

static const char* folder_closed_xpm[]={
  "16 16 9 1",
  "g c #808080",
  "b c #c0c000",
  "e c #c0c0c0",
  "# c #000000",
  "c c #ffff00",
  ". c None",
  "a c #585858",
  "f c #a0a0a4",
  "d c #ffffff",
  "..###...........",
  ".#abc##.........",
  ".#daabc#####....",
  ".#ddeaabbccc#...",
  ".#dedeeabbbba...",
  ".#edeeeeaaaab#..",
  ".#deeeeeeefe#ba.",
  ".#eeeeeeefef#ba.",
  ".#eeeeeefeff#ba.",
  ".#eeeeefefff#ba.",
  ".##geefeffff#ba.",
  "...##gefffff#ba.",
  ".....##fffff#ba.",
  ".......##fff#b##",
  ".........##f#b##",
  "...........####."};

static const char* folder_open_xpm[]={
  "16 16 11 1",
  "# c #000000",
  "g c #c0c0c0",
  "e c #303030",
  "a c #ffa858",
  "b c #808080",
  "d c #a0a0a4",
  "f c #585858",
  "c c #ffdca8",
  "h c #dcdcdc",
  "i c #ffffff",
  ". c None",
  "....###.........",
  "....#ab##.......",
  "....#acab####...",
  "###.#acccccca#..",
  "#ddefaaaccccca#.",
  "#bdddbaaaacccab#",
  ".eddddbbaaaacab#",
  ".#bddggdbbaaaab#",
  "..edgdggggbbaab#",
  "..#bgggghghdaab#",
  "...ebhggghicfab#",
  "....#edhhiiidab#",
  "......#egiiicfb#",
  "........#egiibb#",
  "..........#egib#",
  "............#ee#"};

static const char * folder_locked[]={
  "16 16 10 1",
  "h c #808080",
  "b c #ffa858",
  "f c #c0c0c0",
  "e c #c05800",
  "# c #000000",
  "c c #ffdca8",
  ". c None",
  "a c #585858",
  "g c #a0a0a4",
  "d c #ffffff",
  "..#a#...........",
  ".#abc####.......",
  ".#daa#eee#......",
  ".#ddf#e##b#.....",
  ".#dfd#e#bcb##...",
  ".#fdccc#daaab#..",
  ".#dfbbbccgfg#ba.",
  ".#ffb#ebbfgg#ba.",
  ".#ffbbe#bggg#ba.",
  ".#fffbbebggg#ba.",
  ".##hf#ebbggg#ba.",
  "...###e#gggg#ba.",
  ".....#e#gggg#ba.",
  "......###ggg#b##",
  ".........##g#b##",
  "...........####."};

static const char * pix_file []={
  "16 16 7 1",
  "# c #000000",
  "b c #ffffff",
  "e c #000000",
  "d c #404000",
  "c c #c0c000",
  "a c #ffffc0",
  ". c None",
  "................",
  ".........#......",
  "......#.#a##....",
  ".....#b#bbba##..",
  "....#b#bbbabbb#.",
  "...#b#bba##bb#..",
  "..#b#abb#bb##...",
  ".#a#aab#bbbab##.",
  "#a#aaa#bcbbbbbb#",
  "#ccdc#bcbbcbbb#.",
  ".##c#bcbbcabb#..",
  "...#acbacbbbe...",
  "..#aaaacaba#....",
  "...##aaaaa#.....",
  ".....##aa#......",
  ".......##......."};

// Static global variables
static QPixmap *folderLocked = 0;
static QPixmap *folderClosed = 0;
static QPixmap *folderOpen = 0;
static QPixmap *fileNormal = 0;

// Filter extensions
QString                   Item::typeExtension[16];
QString 		  Item::oldName;
QDict<unsigned short int> Item::extType;


Item::Item( Item* parent, const QString& filename, const bool dir )
  : QListViewItem( parent ), size(0), isDirectory(dir)
{
  setText(0, filename);
  fullName = parent->fullName + '/' + filename;
  setSelectable(false);
}

// Root directory
Item::Item( QListView* parent, const QString& filename )
  : QListViewItem( parent ), isDirectory(true)
{
  // Init pixmaps
  if ( !folderLocked )
    {
      folderLocked = new QPixmap( folder_locked );
      folderClosed = new QPixmap( folder_closed_xpm );
      folderOpen   = new QPixmap( folder_open_xpm );
      fileNormal   = new QPixmap( pix_file );
    }

  setText(0, filename);
  isHidden = false;
  fullName = filename;
}

// Parse file system and recursively fill the entire QListView
void Item::createDirList(bool progressBar)
{
  QDir dir(fullName);
  dir.setFilter(QDir::TypeMask | QDir::Hidden);

  const QFileInfoList* files = dir.entryInfoList();
  if ( files )
    {
      QFileInfoListIterator it( *files );
      QFileInfo * fi;
      QProgressDialog* scanProgress = NULL;
      
      if (progressBar)
	{
	  // Count number of directories
	  unsigned int numDir = 0;
	  while ( (fi=it.current()) != 0 )
	    {
	      if ( fi->isDir() && (fi->fileName() != ".") && (fi->fileName() != ".."))
		numDir++;
	      ++it;
	    }

	  // Create a progress bar 
	  scanProgress = new QProgressDialog("Scanning " + QString::number(numDir) + " director" + ((numDir>1)?"ies":"y") + "...",
					     "Cancel", numDir, NULL, "Scan progress", true);  
	  scanProgress->setMinimumDuration(500);
	  scanProgress->setProgress(0);
      
	  it.toFirst();
	}
      
      while ( (fi=it.current()) != NULL )
	{
	  if ( (fi->fileName() == ".") || (fi->fileName() == "..") )
	    ;
	  else if ( fi->isDir() )
	    {
	      if (scanProgress)
		{
		  scanProgress->setProgress(scanProgress->progress() + 1);
		  qApp->processEvents();
		  if (scanProgress->wasCancelled())
		    qApp->closeAllWindows();
		}
	      
	      Item* dirItem = new Item(this, fi->fileName(), true);
	      
	      if (dir.cd(fi->fileName()))
		{
		  dirItem->setExpandable(true);
		  dirItem->setPixmap( 0, *folderClosed );
		  dirItem->createDirList(false);
		  dir.cdUp();
		}
	      else
		{
		  dirItem->setExpandable(false);
		  dirItem->setPixmap( 0, *folderLocked );
		}
	    }
	  else
	    {
	      // Regular file
	      Item *item = new Item(this, fi->fileName(), false);
	      item->type = Item::fileType(fi);
	      item->setPixmap(0, *fileNormal);
	      item->size = fi->size();
	    }

	  ++it;
	}
      
      if (scanProgress)
	{
	  scanProgress->setProgress(scanProgress->totalSteps());
	  delete scanProgress;
	}
    }
}

void Item::invalidateSizeStrings()
{
  setText(1, QString::null);

  Item* it = (Item*)(firstChild());
  
  // Stop as soon as we descend in a non already open directory
  if (it && it->text(1).isNull())
    return;

  while (it)
    {
      if (it->isDirectory)
	it->invalidateSizeStrings();
      else
	it->setText(1, QString::null);
      
      it = (Item*)(it->nextSibling());
    }
}

// Update the contents of a directory Item. Scans disk.
void Item::rescanDisk()
{
  if (!isDirectory)
    return;
  
  // Delete all children
  Item* it = (Item*)(firstChild());
  while (it)
    {
      Item* next = (Item*)(it->nextSibling());
      delete it;
      it = next;
    }

  // Scan contents
  listView()->setUpdatesEnabled( false );
#if QT_VERSION >= 0x030100
  int sortColumn = listView()->sortColumn();
#else
  int sortColumn = 1;
#endif
  listView()->setSorting( -1 );
  listView()->clearSelection();

  addToParentSizes(false);
  createDirList(true);
  invalidateSizeStrings();
  updateSize();
  setOpen(isOpen());
  addToParentSizes(true);

  listView()->setSorting( sortColumn );
  listView()->setUpdatesEnabled( true );
  ((DirectoryView*)listView())->fitScale();
  listView()->triggerUpdate();
}

short Item::fileType(const QFileInfo* const fi)
{
  QString ext = fi->extension(false);
  if ((ext == "gz") || (ext == "bz2") || (ext == "zip"))
    ext = QFileInfo(fi->baseName(true)).extension(false);
  const unsigned short int* res = extType[ext];
  if (res)
    return *res;
  else
    if (fi->fileName() == "core")
      return 14;
  else
    if (fi->isExecutable())
      return 12;
    else
      return 15;
}

QString Item::stringOfSize(QDU_UINT64 size)
{
  QString res;
  if ( size > (QDU_UINT64)(1024) * (unsigned long int)(1024*1024*10) )
    res.QString::sprintf("%3ld Gb", (unsigned long int)( size/1024/1024/1024) );
  else 
    if (size > 1024*1024*1024)
      res.QString::sprintf("%3.1f Gb", size/1024.0/1024.0/1024.0 );
    else 
      if (size > 1024*1024*10)
	res.QString::sprintf("%3ld Mb", (unsigned long int)(size/1048576) );
      else 
	if (size > 1024*1024)
	  res.QString::sprintf("%3.1f Mb", size/1024.0/1024.0 );
	else 
	  if (size > 1024*10)
	    res.QString::sprintf("%3ld Kb", (unsigned long int)(size/1024) );
	  else
	    if (size > 1024)
	      res.QString::sprintf("%3.1f Kb", size/1024.0 );
	    else
	      res.QString::sprintf("%3ld b", (unsigned long int)(size) );
  return res;
}

void Item::paintCell(QPainter* p, const QColorGroup& cg, int column, int width, int align)
{
  static const int shift = listView()->treeStepSize();
  
  if (column != 2)
    QListViewItem::paintCell(p, cg, column, width, align);
  else
    {
      p->setBrush(listView()->viewport()->backgroundBrush());
      p->setPen(listView()->viewport()->eraseColor());
      p->drawRect ( 0, 0, width, height() );
      if (depth() == 0)
	return;
      int length = (int)(size / ((DirectoryView*)listView())->scale());
      int xoffset = depth()*shift;
      if (length+xoffset >= width)
	length = width;
      int yoffset = depth()+3;
      int hei = height() - 2*yoffset;
      QColor col;
      col.setHsv(210-depth()*shift, 190, 128);
      p->setBrush(col);
      p->setPen(QColor(0,0,0));
      p->drawRect ( xoffset, yoffset, length, hei );
      for (unsigned short i=1 ; i<depth(); ++i)
	p->drawLine(i*shift+10,0,i*shift+10,height());
    }
}

int Item::width( const QFontMetrics& fm, const QListView * lv, int c ) const
{
  if (c!=2)
    return QListViewItem::width(fm,lv,c);
  else
    return lv->columnWidth(2);
}

int Item::compare( QListViewItem* i, int col, bool ascending) const
{
  if (col==0)
    {
      bool dir = ((Item*)i)->isDirectory;
      if (isDirectory == dir)
	return QString::localeAwareCompare( fullName.upper(), (((Item*)i)->fullName).upper() );
      else
	if (ascending)
	  return isDirectory?-1:1;
	else
	  return isDirectory?1:-1;
    }
  else
    {
      QDU_UINT64 s = ((Item*)i)->size;
      // A simple difference is not sufficient as we're dealing with QDU_UINT64.
      if (s > size)
	return 1;
      else if (s == size)
	return 0;
      else
	return -1;
    }
}

bool Item::shouldBeVisible() const
{
  return( ( !isHidden || ((DirectoryView*)listView())->showHidden() ) &&
	  ( size >= ((DirectoryView*)listView())->minThreshold() )  &&
	  ( isDirectory || !((DirectoryView*)listView())->typeFilter() || (type == ((DirectoryView*)listView())->typeFilter())) );
}

void Item::addToParentSizes(bool add) const
{
  Item* it = (Item*)(parent());
  while (it)
    {
      if (add)
	it->size += size;
      else
	it->size -= size;
      it->setText(1, stringOfSize(it->size));
      it->sort();
      
      it = (Item*)(it->parent());
    }
}

void Item::startRename( int col )
{
  if (col == 0)
    {
      Item::oldName = text(0);
      QListViewItem::startRename(col);
    }
}

void Item::okRename( int col )
{
  QListViewItem::okRename(col);

  if ( (col == 0) && (text(0) != Item::oldName) )
    if (!((DirectoryView*)listView())->renameItem(this))
      setText(0, Item::oldName);
}

// Fully recursive computation of directory sizes depending on file filter
void Item::updateSize()
{
  size = 0;
  Item* it = (Item*)(firstChild());
  while (it)
    {
      if (it->isDirectory)
	{
	  it->updateSize();
	  size += it->size;
	}
      else
	if (!((DirectoryView*)listView())->typeFilter() || (it->type == ((DirectoryView*)listView())->typeFilter()))
	  size += it->size;

      it = (Item*)(it->nextSibling());
    }  
}

// Recursively updates visibilty, limited to open directories
void Item::updateVisibility(bool open)
{
  // Delayed Item initialization
  if (text(1).isNull())
    {
      isHidden = (text(0)[0] == '.');
      setText(1, stringOfSize(size));
      setRenameEnabled(0, true);
    }

  Item* it = (Item*)(firstChild());
  while (it)
    {
      if (it->isDirectory && it->isOpen())
	it->updateVisibility(open);

      if (open)
	{
	  // Delayed Item initialization
	  if (it->text(1).isNull())
	    {
	      it->isHidden = (it->text(0)[0] == '.');
	      it->setText(1, stringOfSize(it->size));
	      it->setRenameEnabled(0, true);
	    }

	  it->setVisible( it->shouldBeVisible() );
	}

      it->setSelectable(open);
	    
      it = (Item*)(it->nextSibling());
    }
}

void Item::setOpen( bool open )
{
  listView()->setUpdatesEnabled( false );  
  if (isDirectory)
    if ( open )
      {
	setPixmap( 0, *folderOpen );
	updateVisibility(true);
	sort();
      }
    else
      {
	setPixmap( 0, *folderClosed );
	updateVisibility(false);
      }
  
  QListViewItem::setOpen( open );
  listView()->setUpdatesEnabled( true );
  listView()->triggerUpdate();
}
