/****************************************************************************

 This file is part of the qdu distribution
 Copyright (C) 2002, 2003, 2004  Gilles Debunne (Gilles.Debunne@imag.fr)
 Version 2.2-2, packaged on Tuesday February 1, 2005.

 http://artis.imag.fr/Membres/Gilles.Debunne/Code/QDU

 QDU is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 QDU is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with QDU; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifndef INCLUDE_ITEM_H
#define INCLUDE_ITEM_H

#include <qlistview.h>
#include <qdict.h>
#include <qglobal.h>

typedef unsigned long long int QDU_UINT64;

class QFileInfo;

class Item : public QListViewItem
{
public:
  Item( Item* parent, const QString& filename, const bool dir );
  Item( QListView* parent, const QString& filename );
  
  void setOpen( bool );
  void createDirList( bool progressBar );
  void rescanDisk();
  void invalidateSizeStrings();
  
  void updateSize();
  void updateVisibility(bool open=true);
  bool shouldBeVisible() const;
  void addToParentSizes(bool add) const;
  
  virtual void startRename( int col );
  virtual void okRename( int col );
  virtual void paintCell(QPainter* p, const QColorGroup& cg, int column, int width, int align);
  virtual int width(const QFontMetrics & fm, const QListView* lv, int c) const;
  virtual int compare(QListViewItem* i, int col, bool ascending) const;
    
  static short fileType(const QFileInfo* const fi);
  static QString stringOfSize(QDU_UINT64 size);

  QString fullName;
  QDU_UINT64 size;
  bool isDirectory;
  bool isHidden;
  short type;

  // File and directories renaming
  static QString oldName;;

  // File type filters
  static QString typeExtension[16];
  static QDict<unsigned short int> extType;
};

#endif // INCLUDE_ITEM_H
