/****************************************************************************

 This file is part of the qdu distribution
 Copyright (C) 2002, 2003, 2004  Gilles Debunne (Gilles.Debunne@imag.fr)
 Version 2.2-2, packaged on Tuesday February 1, 2005.

 http://artis.imag.fr/Membres/Gilles.Debunne/Code/QDU

 QDU is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 QDU is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with QDU; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#include "directoryView.h"
#include <qmessagebox.h>
#include <qdir.h>

void DirectoryView::beginUndoEvent()
{
  if (undoIndex_ < undoHistory_.size())
    undoHistory_[undoIndex_].clear();
  else
    {
      QValueVector<UndoEvent> undoVector;
      undoHistory_.push_back(undoVector);
    }
  
  undoIndex_++;
  maxUndoIndex_ = undoIndex_;
}

void DirectoryView::endUndoEvent()
{
  // In case no event was performed, simply skip
#if QT_VERSION >= 0x030100
  if (undoHistory_[undoIndex_-1].isEmpty())
#else
  if (undoHistory_[undoIndex_-1].empty())
#endif
    {
      undoIndex_--;
      maxUndoIndex_ = undoIndex_;
    }
}

void DirectoryView::addEventInUndoHistory(QDropEvent::Action a, Item* sd, Item* s, Item* td, const QString& pn, const QString& nn)
{
  DirectoryView::UndoEvent undoEvent;

  undoEvent.action 	= a;
  undoEvent.sourceDir 	= sd;
  undoEvent.source 	= s;
  undoEvent.targetDir 	= td;
  undoEvent.prevName	= pn;
  undoEvent.newName	= nn;

  undoHistory_[undoIndex_-1].push_back(undoEvent);
}

void DirectoryView::undo()
{
  if (undoIndex_ > 0)
    undoIndex_--;
  else
    {
      QMessageBox::warning(this, "No more undo", "No more undo in history");
      return;
    }

  clearSelection();
  
  QValueVector<UndoEvent> undoVector = undoHistory_[undoIndex_];

  for (QValueVector<UndoEvent>::iterator it = undoVector.begin(); it != undoVector.end(); ++it )
    {
      if ( performAction((*it).action, (*it).targetDir, (*it).source, (*it).sourceDir, (*it).prevName) )
	(*it).source->setSelected(true);
      else
	QMessageBox::warning(this, "Undo error", "Unable to undo " + actionString((*it).action) + "<br>" + (*it).source->fullName
			     + "<br>to<br>" + (*it).sourceDir->fullName + QDir::separator() + (*it).prevName, "Ok");
    }

  updateSelectedItemsList();
}

void DirectoryView::redo()
{
  if (undoIndex_ >= maxUndoIndex_)
    {
      QMessageBox::warning(this, "No more redo", "No more redo is possible");
      return;
    }

  clearSelection();
  
  QValueVector<UndoEvent> undoVector = undoHistory_[undoIndex_];

  for (QValueVector<UndoEvent>::iterator it = undoVector.begin(); it != undoVector.end(); ++it )
    if (! performAction((*it).action, (*it).sourceDir, (*it).source, (*it).targetDir, (*it).newName))
      QMessageBox::warning(this, "Undo error", "Unable to redo " + actionString((*it).action) + "<br>" + (*it).source->fullName
			   + "<br>to<br>" + (*it).targetDir->fullName + QDir::separator() + (*it).newName, "Ok");

  undoIndex_++;
}
